/*
    PPKeyCancellableWindow.m

    Copyright 2013-2016 Josh Freeman
    http://www.twilightedge.com

    This file is part of PikoPixel for Mac OS X and GNUstep.
    PikoPixel is a graphical application for drawing & editing pixel-art images.

    PikoPixel is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License version 3,
    as published by the Free Software Foundation.

    PikoPixel is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#import "PPKeyCancellableWindow.h"

#import "PPKeyConstants.h"
#import "NSObject_PPUtilities.h"


#define kHiddenButtonFrame              NSMakeRect(-1,-1,1,1)


@interface PPKeyCancellableWindow (PrivateMethods)

- (void) addHiddenButtonWithKeyEquivalent: (NSString *) keyEquivalent
                        modifierMask: (unsigned int) modifierMask
                        actionSelector: (SEL) actionSelector;

- (void) performCancelButtonClick: (id) sender;
- (void) performCancelButtonClickAndTerminate: (id) sender;

@end

@implementation PPKeyCancellableWindow

#pragma mark NSWindow overrides

- (void) awakeFromNib
{
    [self addHiddenButtonWithKeyEquivalent: @"w"
            modifierMask: NSCommandKeyMask
            actionSelector: @selector(performCancelButtonClick:)];

    [self addHiddenButtonWithKeyEquivalent: @"."
            modifierMask: NSCommandKeyMask
            actionSelector: @selector(performCancelButtonClick:)];

    [self addHiddenButtonWithKeyEquivalent: kEscKey
            modifierMask: 0
            actionSelector: @selector(performCancelButtonClick:)];

    [self addHiddenButtonWithKeyEquivalent: @"q"
            modifierMask: NSCommandKeyMask
            actionSelector: @selector(performCancelButtonClickAndTerminate:)];
}

#pragma mark Private methods

- (void) addHiddenButtonWithKeyEquivalent: (NSString *) keyEquivalent
                        modifierMask: (unsigned int) modifierMask
                        actionSelector: (SEL) actionSelector
{
    NSButton *button;

    if (![keyEquivalent length] || !actionSelector)
    {
        goto ERROR;
    }

    button = [[[NSButton alloc] initWithFrame: kHiddenButtonFrame] autorelease];

    if (!button)
        goto ERROR;

    [button setKeyEquivalentModifierMask: modifierMask];
    [button setKeyEquivalent: keyEquivalent];

    [button setTarget: self];
    [button setAction: actionSelector];

    [[self contentView] addSubview: button];

    return;

ERROR:
    return;
}

- (void) performCancelButtonClick: (id) sender
{
    [_cancelButton performClick: self];
}

- (void) performCancelButtonClickAndTerminate: (id) sender
{
    [self performCancelButtonClick: self];

    [NSApp ppPerformSelectorFromNewStackFrame: @selector(terminate:)];
}

@end
